import { Account, NetworkType } from 'symbol-sdk';
import { ConfigPreset, NodePreset } from '../model';
import { Addresses, ConfigAccount, NodeAccount } from '../model';
export declare enum Preset {
    bootstrap = "bootstrap",
    testnet = "testnet",
    light = "light"
}
export interface ConfigParams {
    reset: boolean;
    preset: Preset;
    target: string;
    assembly?: string;
    customPreset?: string;
}
export interface ConfigResult {
    addresses: Addresses;
    presetData: ConfigPreset;
}
export declare class ConfigService {
    private readonly root;
    private readonly params;
    static defaultParams: ConfigParams;
    constructor(root: string, params: ConfigParams);
    toConfig(account: Account): ConfigAccount;
    static getNetworkIdentifier(networkType: NetworkType): string;
    static getNetworkName(networkType: NetworkType): string;
    generateAddresses(networkType: NetworkType, size: number): ConfigAccount[];
    generateNodeAccount(index: number, node: NodePreset, networkType: NetworkType): Promise<NodeAccount>;
    generateNodeAccounts(networkType: NetworkType, nodes: NodePreset[]): Promise<NodeAccount[]>;
    generateAddress(networkType: NetworkType): ConfigAccount;
    private static getArray;
    run(): Promise<ConfigResult>;
    private completePresetDataWithRandomConfiguration;
    private generateRandomConfiguration;
    private generateNodes;
    private generateNodeConfiguration;
    private generateP2PFile;
    private generateNemesisConfig;
    private createVrfTransaction;
    private storeTransaction;
    private generateGateways;
}
